from algora.common.base_enum import BaseEnum


class ProjectType(BaseEnum):
    RESEARCH_TEMPLATE = "RESEARCH_TEMPLATE"
    STRATEGY_TEMPLATE = "STRATEGY_TEMPLATE"
    RESEARCH = "RESEARCH"
    STRATEGY = "STRATEGY"
    UNKNOWN = "UNKNOWN"
